package com.biganto.visual.roompark.data.data_provider

import com.biganto.visual.androidplayer.data.repository.local.ILocalStore
import com.biganto.visual.roompark.data.repository.api.IRoomParkApi
import com.biganto.visual.roompark.data.repository.db.IDb
import com.biganto.visual.roompark.data.repository.mapper.fromRaw
import com.biganto.visual.roompark.data.service.user_session.IUserSession
import com.biganto.visual.roompark.domain.contract.FeedsContract
import com.biganto.visual.roompark.domain.model.ArticlesPreviewModel
import com.biganto.visual.roompark.domain.model.FeedModel
import com.biganto.visual.roompark.domain.model.fromEntity
import io.reactivex.Observable
import io.reactivex.Single
import io.reactivex.schedulers.Schedulers
import timber.log.Timber
import timber.log.Timber.e
import javax.inject.Inject

/**
 * Created by Vladislav Bogdashkin on 29.10.2019.
 */

//@Singleton
class FeedsContractModule @Inject constructor(
    private val api: IRoomParkApi,
    private val db: IDb,
    private val local:ILocalStore,
    private val session:IUserSession
): FeedsContract {



    override fun fetchFeeds(): Observable<List<FeedModel>> =
        fetchAllFeeds(session.token)
            .doOnError (::e)


    override fun fetchFeedObservable(id: Int): Observable<ArticlesPreviewModel> {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    override fun getArticle(id: Int): Single<ArticlesPreviewModel> {
        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
    }

    init {
        Timber.d("Feeds Repository Created")
    }


    private fun fetchFeedsApi(token:String) =
        api.getFeeds(token)
            .doOnNext { Timber.d("raw0 $it") }
            .map(::fromRaw)
            .doOnNext(db::blockingUpsert)
            .subscribeOn(Schedulers.io())

    private val fetchFeedsDb =
        db.fetchFeeds()
            .toList()
            .toObservable()
            .subscribeOn(Schedulers.io())

    fun fetchAllFeeds(userToken:String): Observable<List<FeedModel>> =
        Observable.mergeDelayError(
            arrayListOf(fetchFeedsApi(userToken),fetchFeedsDb)
        ).map { fromEntity(it,::fromEntity) }

}

